@@ -11,10 +11,34 @@ Usage:
 
 Plans: 
     - Simple to use functions in easy to read scriptfiles in the style of the ministory file. 
-    - char.check() should take a skill name instead of a value. -> i.e.: char.skill_check("cooking", MW=9), MW optional. 
+    - char.compete(other, skill_name) -> See who wins and by how much. 
 
 Ideas: 
     - Lazy loading modules, to be able to print stuff at once without having to print before the imports.
+    - Add getting experience for groups and show the chars together (only one experience header instead of one per char). 
+
+
+Basic design principles for the scripting language: 
+    
+    - The action is character centered wherever possible and useful. 
+       -> char.say(text) instead of dialog(char, text)
+    
+    - Anything which affects only one character or any interaction between only a few characters which is initiated by one of them gets called from the character via char.action(). 
+       -> char.compete_skill(char2, skill_name) instead of competition_skill(char1, char2, skill_name)
+    
+    - Anything which affects the whole scene, or a whole group of not necessarily interacting characters gets called as basic function via action() or as class in its own right via class.action(). 
+       -> save([char1, char2]) instead of char1.save() char2.save()
+    
+    - The seperate class way should only be chosen, if the class can feel like a character in its own right and needs seperate states which may or may not be persistent over subsequent runs. 
+       -> For example AI.choose_the_way(players_answer) or music.action()
+    
+    - Data should be stored inside the chars wherever possible. If a script gets started with the same character again, the situation should resemble the previous one as much as possible, except where dictated otherwise by the story. 
+       -> char.save() instead of 'on quit' store_char_data(char) + 'on start' load_char_data(char)
+    
+    - Actions should be written as verb_noun or simply verb. 
+       -> char.say() and char.compete_skill() instead of char.text() and char.skill_compete()
+
+The code for the TextRPG can be found at U{http://dreehg.org/ArneBab/textrpg}
 
 """
 
@@ -43,7 +67,7 @@ __copyright__ = """
   MA 02110-1301 USA
 
 """ 
-__version__   = '0.2' 
+__version__   = '0.3' 
 __author__    = 'Arne Babenhauserheide' 
 # __date__      = '7th March 2007' 
 __url__       = 'http://rpg-1d6.sf.net' 
@@ -54,7 +78,7 @@ print "...Loading rpg library..."
 from textrpg_1d6.char import Char as ews_char
 
 def _(text):
-    '''String function to allow localizing later on)'''
+    '''String function to allow localizing later on. '''
     return str(text)
 
 class Char(ews_char):
@@ -79,16 +103,15 @@ class Char(ews_char):
                 diag(self.name + ': ' + i, localize=False)
             # If the string is empty, just add a blank line without the characters name. 
             else: diag("...", localize=False)
-            
-    def skill_check(skill_name, mods=[], MW=9): 
-        """Test the skill of the char against a target number."""
-        if skill_name in self.skills: 
-            value = self.skills[skill_name][_("Zahlenwert")]
-        else: 
-            # If the char doesn't have the skill, we use the default. ouch! 
-            # TODO: Query a list of related skills (can be used with a mod of -3). 
-            value = 3
-        return super(Char, self).check(value, mods=mods, MW=MW)
+        
+    def compete_skill(self, other, skill_name, self_mods=[], other_mods=[]): 
+        """Compete with the other char in the specified skill. 
+        
+        @return If we won and by how much: (won, diff). Diff is always >=0. 
+        """
+        my_result = self.roll(self.get_effective_skill_value(skill_name, related_skills=[], related_attributes=[]), mods=self_mods)
+        other_result = other.roll(other.get_effective_skill_value(skill_name, related_skills=[], related_attributes=[]), mods=other_mods)
+        return my_result > other_result, abs(my_result - other_result)
     
     def fight_round(self, other, *args, **kwds): 
         """Fight for one round."""
@@ -97,6 +120,10 @@ class Char(ews_char):
     def battle(self, other): 
         """Fight a dadly battle."""
         return battle(self, other)
+    
+    def get_exp(self, amount=0): 
+        """Get experience and show it."""
+        return give_exp(self, amount)
         
         
     def fight_round(self,  other, styles_self=[], styles_other=[]):
@@ -313,7 +340,7 @@ def select_battle_action(me, other, attack=True):
     
     @return: The result of the battle (won or lost, wounds)."""
     
-    if ask('Do you want to attack ' + other.name + '? (Yes, no) ').lower() in ['Yes', 'y', '']: 
+    if ask('Do you want to attack ' + other.name + '? (Yes, no) ').lower() in ['yes', 'y', '']: 
         diag("You attack " + other.name + ".")
         won, injuries_self, injuries_other = select_battle_style(me, other, attacker=True)
     else:
@@ -379,22 +406,54 @@ def fight_while_standing(me, other):
     else: 
         return False # We lost
 
-def give_exp(char, amount=9): 
-    """Wrapper for scripts."""
-    return get_experience(char, amount=amount)
+def give_exp(char, amount=0): 
+    """Give experience to only one char."""
+    return get_experience([char], amount=amount)
 
-def get_experience(char, amount=0):
-    """Show the experience the char gets and what the char does with it."""
-    diag("\n***experience of " + char.name + "***")
-    diag(char.name + " got " + str(amount) + " experience.")
+def get_experience(chars, amount=0):
+    """Show the experience the chars get and what they do with it."""
+    # Dialog header
+    if len(chars) == 1: 
+        diag("\n***experience of " + chars[0].name + "***")
+        diag(chars[0].name + " got " + str(amount) + " experience.")
+    else: 
+        diag("\n***experience***")
+        diag("The characters got " + str(amount) + " experience.")
+    
+    # Info text for each char. 
+    for i in chars: 
+        # Upgrade in single point steps, so the experience gets spread a bit. 
+        # if it's less than two points, use it completely. 
+        if amount <= 2: 
+            text = upgrade_info(i, amount)
+            if text: 
+                diag(text)
+        # If it's more than one point, spread all but one in single points 
+        # and at last improve by 1 plus the remaining fraction. 
+        # With this, no value gets less than one point/Strich 
+        # (no use in spreading points which can only give an increase by pure chance). 
+        else: 
+            for j in range(int(amount) -1): 
+                text = upgrade_info(i, 1)
+                if text: 
+                    diag(text)
+            text = upgrade_info(i, amount - int(amount) + 1)
+            if text: 
+                diag(text)
+        
+    # Dialog footer. 
+    diag("***/experience***\n")
+
+def upgrade_info(char, amount): 
+    """Return the effects of improving teh char by the given amount of points/Strichen."""
     upgrade = char.upgrade(amount)
     item_name = " ".join(upgrade[0].items()[0][0])
     old_value = upgrade[0].items()[0][1]["Zahlenwert"] 
     new_value = upgrade[1].items()[0][1]["Zahlenwert"]
     if old_value < new_value:
-        diag(char.name + " raised " + item_name + " from " + str(old_value) + " to " + str(new_value))
-    diag("***/experience***\n")
-
+        return char.name + " raised " + item_name + " from " + str(old_value) + " to " + str(new_value)
+    else: 
+        return False
 
 if __name__ == "__main__": 
     print "...Creating main character..."
